/*
 * This file is part of the SOAFaces Project (hereinafter "SOAFaces").
 * Copyright (C) 2006 Grand Logic, Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package org.soafaces.bundle.client.rpc;

import org.soafaces.services.client.rpc.UniversalClient;;

/**
 * Context associated with the running of the IOViewer
 * and the Tasklet that it is associated with. The
 * input and output JavaBeans reflect the input and output of the
 * a Tasklet workflow when it was run for the given run workflow run ID.
 *
 * @author Sam Taha
 */
 
public interface IOViewerContext {
  
  /**
   * Unique id assigned to the workflow/job that ran the Tasklet that
   * the RuntimeViewer is associated with.
   * @return
   */
  long getJobRunID();
  
  /**
   * Unique id of workflow/job that is associatd with this RuntimeViewer.
   * 
   * @return -1 if not a Weblet and not associated with a workflow.
   */
  long getJobID();
  
  /**
   * User defined name of workflow/job that is associatd with this RuntimeViewer.
   * 
   * @return Empty string if Weblet not associated with a workflow.
   */
  String getJobName();
  
  /**
   * Unique id of Tasklet that the RuntimeViewer is associated with.
   * @return
   */
  long getTaskletID();
  
  /**
   * User defined name of the Tasklet that the RuntimeViewer is associated with.
   * @return
   */
  String getTaskletName();
  
  /**
   * Username of user accessing this IOViewer.
   * 
   * @return  Empty string if not defined.
   */
  String getUsername();

  /**
   * Get the <code>TaskletStatus</code> object associated with the running of a Tasklet from the server
   * This will contain updated information on running state of the Tasklet (if it is still
   * running). To get updated status from the Tasklet, you must call this method to get
   * the latest <code>TaskletStatus</code> object.
   *
   * <ul>
   *  The posssible <code>FetchPOJOCallback.onFailure()</code> exceptions that can be returned
   *  if there is a problem include:
   *  <li><code>BeanTransferException</code> when there is a server-side problem
   *    fetching the input JavaBean.</li>
   *  <li>Standard RPC exceptions that can be thrown by GWT.</li>
   * </ul>
   *
   * @param callback Return TaskletStatus object.
   */
  void fetchTaskletStatus(FetchTaskletStatusCallback callback);
  
  /**
   * Get the input JavaBean associated with the running of a Tasklet from the server.
   * 
   * <ul>
   *  The posssible <code>FetchPOJOCallback.onFailure()</code> exceptions that can be returned
   *  if there is a problem include:
   *  <li><code>BeanTransferException</code> when there is a server-side problem
   *    fetching the input JavaBean.</li>
   *  <li>Standard RPC exceptions that can be thrown by GWT.</li>
   * </ul>
   * 
   * @param callback Return results of input JavaBean as a POJO.
   */
  void fetchInputBean(FetchPOJOCallback callback);
  
  /**
   * Get the input JavaBean associated with the running of a Tasklet from the server.
   * 
   * <ul>
   *  The posssible <code>FetchJSONCallback.onFailure()</code> exceptions that can be returned
   *  if there is a problem include:
   *  <li><code>BeanTransferException</code> when there is a server-side problem
   *    fetching the input JavaBean.</li>
   *  <li>Standard RPC exceptions that can be thrown by GWT.</li>
   * </ul>
   * 
   * @param callback Return results of input JavaBean as a JSON.
   */
  void fetchInputBean(FetchJSONCallback callback);
  
  /**
   * Get the output JavaBean associated with the running of a Tasklet from the server.
   * 
   * <ul>
   *  The posssible <code>FetchPOJOCallback.onFailure()</code> exceptions that can be returned
   *  if there is a problem include:
   *  <li><code>BeanTransferException</code> when there is a server-side problem
   *    fetching the output JavaBean.</li>
   *  <li>Standard RPC exceptions that can be thrown by GWT.</li>
   * </ul>
   * 
   * @param callback Return results of input JavaBean as a POJO.
   */
  void fetchOutputBean(FetchPOJOCallback callback);
  
  /**
   * Get the output JavaBean associated with the running of a Tasklet from the server.
   * 
   * <ul>
   *  The posssible <code>FetchJSONCallback.onFailure</code> exceptions that can be returned
   *  if there is a problem include:
   *  <li><code>BeanTransferException</code> when there is a server-side problem
   *    fetching the output JavaBean.</li>
   *  <li>Standard RPC exceptions that can be thrown by GWT.</li>
   * </ul>
   * 
   * @param callback Return results of input JavaBean as a JSON.
   */
  void fetchOutputBean(FetchJSONCallback callback);
  
  /**
   * Messaging client that can be used to access Mule endpoint services and SOAFaces
   * endpoint services.
   * @return
   */
  UniversalClient getUniversalClient();
}
